@@ -73,9 +73,12 @@ names.vctrs_rcrd <- function(x) {
 
 #' @export
 format.vctrs_rcrd <- function(x, ...) {
-  # nocov start
-  stop_unimplemented(x, "format")
-  # nocov end
+  if (inherits(x, "vctrs_foobar")) {
+    # For unit tests
+    exec("paste", !!!vec_data(x), sep = ":")
+  } else {
+    stop_unimplemented(x, "format")
+  }
 }
 
 #' @export
@@ -136,11 +139,6 @@ rep.vctrs_rcrd <- function(x, ...) {
   vec_restore(out, x)
 }
 
-#' @export
-as.list.vctrs_rcrd <- function(x, ...) {
-  vec_cast(x, list())
-}
-
 # Replacement -------------------------------------------------------------
 
 #' @export
