@@ -1,4 +1,5 @@
 #include "vctrs.h"
+#include "c.h"
 #include "ptype-common.h"
 #include "slice-assign.h"
 #include "utils.h"
@@ -22,33 +23,76 @@ SEXP vctrs_c(SEXP call, SEXP op, SEXP args, SEXP env) {
   return out;
 }
 
+
 // [[ include("vctrs.h") ]]
 SEXP vec_c(SEXP xs,
            SEXP ptype,
            SEXP name_spec,
            const struct name_repair_opts* name_repair) {
-  R_len_t n = Rf_length(xs);
+  struct fallback_opts opts = {
+    .df = DF_FALLBACK_DEFAULT,
+    .s3 = S3_FALLBACK_true
+  };
+  return vec_c_opts(xs, ptype, name_spec, name_repair, &opts);
+}
 
-  if (needs_vec_c_fallback(xs, ptype)) {
-    return vec_c_fallback(xs, name_spec);
-  }
-
-  ptype = PROTECT(vec_ptype_common_params(xs, ptype, DF_FALLBACK_DEFAULT));
+SEXP vec_c_opts(SEXP xs,
+                SEXP ptype,
+                SEXP name_spec,
+                const struct name_repair_opts* name_repair,
+                struct fallback_opts* fallback_opts) {
+  SEXP orig_ptype = ptype;
+  ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, fallback_opts));
 
   if (ptype == R_NilValue) {
     UNPROTECT(1);
     return R_NilValue;
   }
 
+  if (needs_vec_c_fallback(ptype)) {
+    SEXP out = vec_c_fallback(ptype, xs, name_spec, name_repair);
+    UNPROTECT(1);
+    return out;
+  }
+  // FIXME: Needed for dplyr::summarise() which passes a non-fallback ptype
+  if (needs_vec_c_homogeneous_fallback(xs, ptype)) {
+    SEXP out = vec_c_fallback_invoke(xs, name_spec);
+    UNPROTECT(1);
+    return out;
+  }
+
+  // FIXME: If data frame, recompute ptype without common class
+  // fallback. Should refactor this to allow common class fallback
+  // with data frame columns.
+  //
+  // FIXME: If `ptype` is a `vctrs_vctr` class without a
+  // `vec_ptype2()` method, the common type is a common class
+  // fallback. To avoid infinit recursion through `c.vctrs_vctr()`, we
+  // bail out from `needs_vec_c_fallback()`. In this case recurse with
+  // fallback disabled as well.
+  if ((is_data_frame(ptype) && fallback_opts->s3 == S3_FALLBACK_true) ||
+      vec_is_common_class_fallback(ptype)) {
+    struct fallback_opts d_fallback_opts = *fallback_opts;
+    d_fallback_opts.s3 = S3_FALLBACK_false;
+
+    UNPROTECT(1);
+    ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, &d_fallback_opts));
+    xs = vec_cast_common_opts(xs, ptype, &d_fallback_opts);
+  } else {
+    xs = vec_cast_common_opts(xs, ptype, fallback_opts);
+  }
+  PROTECT(xs);
+
   // Find individual input sizes and total size of output
+  R_len_t n = Rf_length(xs);
   R_len_t out_size = 0;
 
   SEXP ns_placeholder = PROTECT(Rf_allocVector(INTSXP, n));
   int* ns = INTEGER(ns_placeholder);
 
   for (R_len_t i = 0; i < n; ++i) {
-    SEXP elt = VECTOR_ELT(xs, i);
-    R_len_t size = (elt == R_NilValue) ? 0 : vec_size(elt);
+    SEXP x = VECTOR_ELT(xs, i);
+    R_len_t size = (x == R_NilValue) ? 0 : vec_size(x);
     out_size += size;
     ns[i] = size;
   }
@@ -85,12 +129,11 @@ SEXP vec_c(SEXP xs,
     }
 
     SEXP x = VECTOR_ELT(xs, i);
-    SEXP elt = PROTECT(vec_cast(x, ptype, args_empty, args_empty));
 
     init_compact_seq(idx_ptr, counter, size, true);
 
     // Total ownership of `out` because it was freshly created with `vec_init()`
-    out = vec_proxy_assign_opts(out, idx, elt, vctrs_ownership_total, &c_assign_opts);
+    out = vec_proxy_assign_opts(out, idx, x, vctrs_ownership_total, &c_assign_opts);
     REPROTECT(out, out_pi);
 
     if (has_names) {
@@ -107,7 +150,6 @@ SEXP vec_c(SEXP xs,
     }
 
     counter += size;
-    UNPROTECT(1);
   }
 
   out = PROTECT(vec_restore(out, ptype, R_NilValue));
@@ -123,14 +165,33 @@ SEXP vec_c(SEXP xs,
     out = vec_set_names(out, R_NilValue);
   }
 
-  UNPROTECT(7);
+  UNPROTECT(8);
   return out;
 }
 
 static inline bool vec_implements_base_c(SEXP x);
 
-// [[ include("vctrs.h") ]]
-bool needs_vec_c_fallback(SEXP xs, SEXP ptype) {
+// [[ include("c.h") ]]
+bool needs_vec_c_fallback(SEXP ptype) {
+  if (!vec_is_common_class_fallback(ptype)) {
+    return false;
+  }
+
+  // Suboptimal: Prevent infinite recursion through `vctrs_vctr` method
+  SEXP class = PROTECT(Rf_getAttrib(ptype, syms_fallback_class));
+  class = r_chr_get(class, r_length(class) - 1);
+
+  if (class == strings_vctrs_vctr) {
+    UNPROTECT(1);
+    return false;
+  }
+
+  UNPROTECT(1);
+  return true;
+}
+
+// [[ include("c.h") ]]
+bool needs_vec_c_homogeneous_fallback(SEXP xs, SEXP ptype) {
   if (!Rf_length(xs)) {
     return false;
   }
@@ -146,11 +207,6 @@ bool needs_vec_c_fallback(SEXP xs, SEXP ptype) {
     return false;
   }
 
-  // Temporary compatibility with `sf` implementations
-  if (Rf_inherits(x, "sfc")) {
-    return false;
-  }
-
   if (ptype != R_NilValue) {
     SEXP x_class = PROTECT(r_class(x));
     SEXP ptype_class = PROTECT(r_class(ptype));
@@ -167,7 +223,9 @@ bool needs_vec_c_fallback(SEXP xs, SEXP ptype) {
     list_is_homogeneously_classed(xs) &&
     vec_implements_base_c(x);
 }
-static inline bool vec_implements_base_c(SEXP x) {
+
+static inline
+bool vec_implements_base_c(SEXP x) {
   if (!OBJECT(x)) {
     return false;
   }
@@ -178,28 +236,66 @@ static inline bool vec_implements_base_c(SEXP x) {
     return s3_find_method("c", x, base_method_table) != R_NilValue;
   }
 }
+static inline
+bool class_implements_base_c(SEXP cls) {
+  if (s3_class_find_method("c", cls, base_method_table) != R_NilValue) {
+    return true;
+  }
+  if (s4_class_find_method(cls, s4_c_method_table) != R_NilValue) {
+    return true;
+  }
+  return false;
+}
 
 static inline int vec_c_fallback_validate_args(SEXP x, SEXP name_spec);
 static inline void stop_vec_c_fallback(SEXP xs, int err_type);
 
-// [[ include("vctrs.h") ]]
-SEXP vec_c_fallback(SEXP xs, SEXP name_spec) {
+// [[ include("c.h") ]]
+SEXP vec_c_fallback(SEXP ptype,
+                    SEXP xs,
+                    SEXP name_spec,
+                    const struct name_repair_opts* name_repair) {
+  SEXP class = PROTECT(Rf_getAttrib(ptype, syms_fallback_class));
+  bool implements_c = class_implements_base_c(class);
+  UNPROTECT(1);
+
+  if (implements_c) {
+    return vec_c_fallback_invoke(xs, name_spec);
+  } else {
+    struct fallback_opts fallback_opts = {
+      .df = DF_FALLBACK_none,
+      .s3 = S3_FALLBACK_false
+    };
+
+    // Should cause a common type error, unless another fallback
+    // kicks in (for instance, homogeneous class with homogeneous
+    // attributes)
+    vec_ptype_common_opts(xs, R_NilValue, &fallback_opts);
+
+    // Suboptimal: Call `vec_c()` again to combine vector with
+    // homogeneous class fallback
+    return vec_c_opts(xs, R_NilValue, name_spec, name_repair, &fallback_opts);
+  }
+}
+
+// [[ include("c.h") ]]
+SEXP vec_c_fallback_invoke(SEXP xs, SEXP name_spec) {
   SEXP x = list_first_non_null(xs, NULL);
 
+  if (vctrs_debug_verbose) {
+    Rprintf("Falling back to `base::c()` for class `%s`.\n",
+            r_chr_get_c_string(r_class(x), 0));
+  }
+
   int err_type = vec_c_fallback_validate_args(x, name_spec);
   if (err_type) {
     stop_vec_c_fallback(xs, err_type);
   }
 
-  SEXP args = PROTECT(Rf_coerceVector(xs, LISTSXP));
-  args = PROTECT(node_compact_d(args));
+  SEXP call = PROTECT(Rf_lang2(Rf_install("base_c_invoke"), xs));
+  SEXP out = Rf_eval(call, vctrs_ns_env);
 
-  SEXP call = PROTECT(Rf_lcons(Rf_install("c"), args));
-
-  // Dispatch in the base namespace which inherits from the global env
-  SEXP out = Rf_eval(call, R_BaseNamespace);
-
-  UNPROTECT(3);
+  UNPROTECT(1);
   return out;
 }
 
