@@ -349,9 +349,8 @@ SEXP vec_slice_impl(SEXP x, SEXP subscript) {
       out = PROTECT_N(vec_slice_dispatch(x, subscript), &nprot);
     }
 
-    // Take over attribute restoration only if the `[` method did not
-    // restore itself
-    if (ATTRIB(out) == R_NilValue) {
+    // Take over attribute restoration only if there is no `[` method
+    if (!vec_is_restored(out, x)) {
       out = vec_restore(out, x, restore_size);
     }
 
@@ -410,6 +409,35 @@ SEXP vec_slice_impl(SEXP x, SEXP subscript) {
   }
 }
 
+bool vec_is_restored(SEXP x, SEXP to) {
+  // Don't restore if there is an actual `[` method that ignored
+  // attributes. Some methods like [.ts intentionally strip the class
+  // and attributes. FIXME: This branch is now probably sufficient.
+  if (s3_find_method("[", to, base_method_table) != R_NilValue) {
+    return true;
+  }
+
+  SEXP attrib = ATTRIB(x);
+
+  if (attrib == R_NilValue) {
+    return false;
+  }
+
+  // Class is restored if it contains any other attributes than names.
+  // We might want to add support for data frames later on.
+  SEXP node = attrib;
+  while (node != R_NilValue) {
+    if (TAG(node) == R_NamesSymbol) {
+      node = CDR(node);
+      continue;
+    }
+    return true;
+  }
+
+  return false;
+}
+
+
 // [[export]]
 SEXP vctrs_slice(SEXP x, SEXP subscript) {
   vec_assert(x, args_empty);
