@@ -185,27 +185,44 @@ SEXP vctrs_type_common(SEXP call, SEXP op, SEXP args, SEXP env) {
   SEXP types = PROTECT(rlang_env_dots_values(env));
   SEXP ptype = PROTECT(Rf_eval(CAR(args), env));
 
-  SEXP out = vec_ptype_common_params(types, ptype, DF_FALLBACK_DEFAULT);
+  SEXP out = vec_ptype_common_params(types, ptype, DF_FALLBACK_DEFAULT, S3_FALLBACK_false);
 
   UNPROTECT(2);
   return out;
 }
 
-SEXP vctrs_ptype_common_params(SEXP call, SEXP op, SEXP args, SEXP env) {
+// [[ register(external = TRUE) ]]
+SEXP vctrs_ptype_common_opts(SEXP call, SEXP op, SEXP args, SEXP env) {
   args = CDR(args);
 
   SEXP types = PROTECT(rlang_env_dots_values(env));
   SEXP ptype = PROTECT(Rf_eval(CAR(args), env)); args = CDR(args);
-  SEXP df_fallback = PROTECT(Rf_eval(CAR(args), env));
+  SEXP opts = PROTECT(Rf_eval(CAR(args), env));
 
-  SEXP out = vec_ptype_common_params(types, ptype, r_int_get(df_fallback, 0));
+  const struct fallback_opts c_opts = new_fallback_opts(opts);
+
+  SEXP out = vec_ptype_common_opts(types, ptype, &c_opts);
 
   UNPROTECT(3);
   return out;
 }
+// [[ include("ptype-common.h") ]]
+SEXP vec_ptype_common_params(SEXP dots,
+                             SEXP ptype,
+                             enum df_fallback df_fallback,
+                             enum s3_fallback s3_fallback) {
+  struct fallback_opts opts = {
+    .df = df_fallback,
+    .s3 = s3_fallback
+  };
+
+  return vec_ptype_common_opts(dots, ptype, &opts);
+}
 
 // [[ include("ptype-common.h") ]]
-SEXP vec_ptype_common_params(SEXP dots, SEXP ptype, enum df_fallback df_fallback) {
+SEXP vec_ptype_common_opts(SEXP dots,
+                           SEXP ptype,
+                           const struct fallback_opts* opts) {
   if (!vec_is_partial(ptype)) {
     return vec_ptype(ptype, args_dot_ptype);
   }
@@ -214,8 +231,11 @@ SEXP vec_ptype_common_params(SEXP dots, SEXP ptype, enum df_fallback df_fallback
     Rf_errorcall(R_NilValue, "strict mode is activated; you must supply complete `.ptype`.");
   }
 
+  // Remove constness
+  struct fallback_opts mut_opts = *opts;
+
   // Start reduction with the `.ptype` argument
-  SEXP type = PROTECT(reduce(ptype, args_dot_ptype, dots, &vctrs_type2_common, &df_fallback));
+  SEXP type = PROTECT(reduce(ptype, args_dot_ptype, dots, &vctrs_type2_common, &mut_opts));
   type = vec_ptype_finalise(type);
 
   UNPROTECT(1);
@@ -228,14 +248,13 @@ static SEXP vctrs_type2_common(SEXP current,
                                struct counters* counters,
                                void* data) {
   int left = -1;
-  enum df_fallback df_fallback = *((enum df_fallback*) data);
 
   const struct ptype2_opts opts = {
     .x = current,
     .y = next,
     .x_arg = counters->curr_arg,
     .y_arg = counters->next_arg,
-    .df_fallback = df_fallback
+    .fallback = *((struct fallback_opts*) data)
   };
 
   current = vec_ptype2_opts(&opts, &left);
